## Administration des services

Chaque service versionné sur ce dépôt est lancé sur une des machines virtuelles de Picasoft.

En particulier, les fichier `docker-compose.yml` présents dans chaque dossier, ainsi que les fichiers de configuration et de secrets, permettent de lancer n'importe quel service de façon autonome, sur n'importe quel machine.

Pour chacune des opérations détaillées dans cette page, on suppose que :
* Vous savez sur quel machine tourne ou doit tourner le service (`pica01`, `monitoring`...)
* Vous avez une connexion SSH active sur cette machine
* Vous êtes dans le dossier `<clone_depot>/<service>`, sur la branche `master`. Le clone devrait se trouver dans `/DATA/docker/dockerfiles`.
* Vous êtes connecté au registre de production : on s'assure de bien avoir exécuté la commande `docker login registry.picasoft.net`. Les identifiants sont sur le [pass](https://gitlab.utc.fr/picasoft/interne/pass).

La branche `master` devant refléter l'état des services en production, toutes les opérations sur les machines de production devront être réalisées depuis `master` : cela signifie que les branches sur lesquelles vous travaillez devront être fusionnées à la fin de vos tests, et donc avant la mise en production.

### Opérations usuelles

Consulter les logs d'un service à la recherche d'une erreur :

```bash
# n est le nombre de lignes à afficher, optionnel
docker-compose logs -f [--tail <n>]
```

Redémarrer un service, par exemple si celui-ci a planté :

```bash
# Le nom du service est optionnel : si on veut tout
# redémarrer, on ne le met pas. Sinon, on indique
# le nom du service présent dans le fichier
# docker-compose.yml.
docker-compose restart [service]
```

Re-créer le conteneur si nécessaire, par exemple suite à une mise à jour de l'image ou de la configuration :

```bash
docker-compose pull
# Le nom du service est optionnel : si on veut tout
# redémarrer, on ne le met pas. Sinon, on indique
# le nom du service présent dans le fichier
# docker-compose.yml.
docker-compose up -d [service]
```

Le `docker-compose pull` est **essentiel** : il permet de récupérer les nouvelles versions des images et de s'assurer qu'on a bien la dernière version indiquée dans le Compose.

Éteindre un service :
```bash
docker-compose stop
```

etc.

Attention : un conteneur noté `Unhealthy` à cause d'un mauvais `HEALTHCHECK` sera **exclu** de Traefik, même s'il fonctionne bien!

### Lancement d'un nouveau service

Lorsque vous lancez un nouveau service, il y a parfois quelques opérations à faire avant de lancer les conteneurs.
En effet, bien qu'en théorie un `docker-compose up -d` suffise, il y a quelques cas particuliers.

#### Secrets

Les secrets, tels que les mots de passe, ne sont évidemment pas présents sur ce dépôt. Lorsque des secrets sont utilisés, ils sont **systématiquement** indiqués par la présence de fichiers dans un sous-répertoire `secrets`.

Dans ce cas, on copiera tous les fichiers `.secrets.example` en `.secrets`, et on remplacera les valeurs.

Cette opération n'est nécessaire que lors du premier lancement.
Il y a plusieurs situations.

1. Le mot de passe est "inédit", et ne concerne pas les utilisateurs (comme le mot de passe d'un utilisateur de base de données). On pourra générer un mot de passe avec la commande suivante :

```bash
< /dev/urandom tr -dc _A-Z-a-z-0-9 | head -c${1:-32};echo;
```

2. Le mot de passe est connu, par exemple le mot de passe d'un compte LDAP. On le récupérera sur le [pass](https://gitlab.utc.fr/picasoft/interne/pass).
3. Le mot de passe est "inédit", mais doit être accessibles aux administrateurs (par exemple le mot de passe d'administration d'Etherpad). On le créera directement dans le [pass](https://gitlab.utc.fr/picasoft/interne/pass) (`picapass insert...`) et on le copiera dans le fichier de secrets.

#### Création des volumes

Si des volumes sont déclarés `external` dans le fichier Compose (ce que nous ne [recommendons pas](./guide_bonnes_pratiques.md)), il faut les créer manuellement au prélable :

```bash
docker volume create <volume>
```

#### Opérations manuelles

Certains services (comme [le serveur mail](../pica-mail)) nécessitent des opérations manuelles : créer des dossiers, puis générer des clés ou des certificats... Il faudra être attentif à le faire avant tout lancement.
