#!/bin/bash

#edition de /etc/postfix/main.cf via un utilitaire fourni par postfix

#identité
#nom de domaine des courriers dont on est à l'origine (pour s'en réserver l'exclusivité)
postconf -e "mydomain = ${MY_DOMAIN}"
postconf -e "myorigin = ${MY_DOMAIN}"
#nom d'\hôte de la machine qui va faire tourner le MTA
postconf -e "myhostname=${MY_HOSTNAME}"

#boîtes mail
#cette configuration permet à Postfix permet de savoir quelles adresses en @picasoft.net existent
#configuration du local delivery agent, ici c'est le MDA Dovecot, on communique avec le protocole LMTP sur TCP/IP (inet dans le jargon Postfix)
postconf -e "virtual_transport = lmtp:inet:${LMTP_LAN_HOSTNAME}:${LMTP_PORT}"
#noms de domaines des boîtes gérées par ce serveur de fin de chaîne
postconf -e "virtual_mailbox_domains = ${MY_DOMAIN}, localhost.${MY_DOMAIN}, localhost"
#map: donne la liste des adresses existantes
cat <<EOF >> /etc/postfix/ldap-virtual-mailbox-maps
server_host = ${LDAP_PROTOCOL}://${LDAP_SERVER_HOSTNAME}:${LDAP_PORT}
search_base = ${LDAP_SEARCH_BASE}
query_filter = ${LDAP_VIRTUAL_MAILBOX_FILTER}
bind = yes
bind_dn = ${LDAP_BIND_DN}
bind_pw = ${LDAP_BIND_PW}
result_attribute = uid
# Mandatory for SSL : http://www.postfix.org/ldap_table.5.html
# Section LDAP SSL AND STARTTLS PARAMETERS
version = 3
EOF
postconf -e "virtual_mailbox_maps = ldap:/etc/postfix/ldap-virtual-mailbox-maps"
postconf -e "smtpd_sender_login_maps = ldap:/etc/postfix/ldap-virtual-mailbox-maps"


#en dessous de ce degré de "vérité", les boîtes non trouvées en table sont rejetées. (100 correspond à un "vrai" simple, mais il est possible de définir différents degrés d'appartenance à la table des utilisateurs dans des configurations plus complexes: une boîte pourrait par exemple avoir le bon nom de domaine mais pas le bon nom d'utilisateur, mais ne pas être rejetée;)
postconf -e "virtual_minimum_uid = 100"
#on fait confiance à l'hôte local, utile pour faire du debug
postconf -e "mynetworks = 127.0.0.0/8"

#restrictions sur les destinataires
#les personnes authentifiées peuvent envoyer des mails à tout le monde, les personnes non authentifiées peuvent envoyer des mails seulement aux destinations connues donc des destinataires en @picasoft.net
#on rejette si le destinataire n'a pas de MX ni de A et n'est pas nous-même
#l'ordre permit_sasl_authenticated, reject_unauth_destination est important: si on est authentifié on fait ce qu'on veut, si on n'est pas authentifié on ne peut qu'envoyer aux destinations connues
#postconf -e "smtpd_recipient_restrictions = reject_unknown_recipient_domain, permit_sasl_authenticated, reject_unauth_destination"
postconf -e 'smtpd_recipient_restrictions = reject_unknown_recipient_domain, permit_sasl_authenticated, reject_unauth_destination'

#restrictions permettant de tester le client dès le helo
#on rejette les clients qui n'ont pas été capables de donner notre nom de domaine lors du helo
#on interdit de faire du helo avec un domaine non fully qualified, sinon c'est trop facile de tenter localhost., mail., pica01. etc.
#on interdit les domaines qui n'ont pas de A ou de MX
postconf -e "smtpd_helo_restrictions = reject_invalid_helo_hostname, reject_non_fqdn_helo_hostname, reject_unknown_helo_hostname"
postconf -e "smtpd_helo_required = yes"

#blacklists basées sur l'adresse du client
postconf -e "smtpd_client_restrictions = ${SMTPD_CLIENT_RESTRICTIONS}"

#on interdit aux gens d'utiliser une adresse qui leur appartient pas
postconf -e "smtpd_sender_restrictions = reject_unauthenticated_sender_login_mismatch reject_authenticated_sender_login_mismatch"



#Saslauthd

#utiliser le démon saslauthd. Il est contacté par des appels de fonction à une lib et retourne la validité des login.
postconf -e "smtpd_sasl_path = smtpd"
cat <<EOF >> /etc/postfix/sasl/smtpd.conf
pwcheck_method: saslauthd
mech_list: PLAIN LOGIN
EOF
#fichiers de config et socket utilisés par le démon de saslauthd créé pour postfix (voir fichier copié dans le Dockerfile)
dpkg-statoverride --add root sasl 710 /var/spool/postfix/var/run/saslauthd
adduser postfix sasl
service saslauthd  restart
#activation du sasl
postconf -e 'smtpd_sasl_local_domain = '
postconf -e 'smtpd_sasl_auth_enable = yes'
#autorise l'auth sasl seulement sur un canal tls
postconf -e 'smtpd_tls_auth_only = yes'

#autorise l'auth depuis des clients connus comme obsolètes/non standard (outlook) mais ne présentant pas de faille de sécurité
postconf -e 'broken_sasl_auth_clients = yes'
#indique que la source des utilisateurs SASL est un serveur LDAP
cat <<EOF >> /etc/default/saslauthd-postfix
MECHANISMS="ldap"
MECH_OPTIONS=""
EOF
#on enlève le démon saslauthd par défaut de debian (nous utilisons le fichier saslauthd-postfix)
#ça ne change rien à part qu'il n'y a pas de warning dans les logs concernant un service non utilisé
rm /etc/default/saslauthd
#configuration de ce serveur LDAP
cat <<EOF >> /etc/saslauthd.conf
ldap_servers: ${LDAP_PROTOCOL}://${LDAP_SERVER_HOSTNAME}:${LDAP_PORT}
ldap_bind_dn: ${LDAP_BIND_DN}
ldap_bind_pw: ${LDAP_BIND_PW}
ldap_search_base: ${LDAP_SEARCH_BASE}
ldap_filter: ${LDAP_SASL_FILTER}
EOF
#prise en compte de la config SASL
service saslauthd restart
service postfix restart

cat <<EOF >> /etc/opendmarc.conf
#ajout de mon nom d'hôte
TrustedAuthservIDs ${MY_HOSTNAME}
AuthservID ${MY_HOSTNAME}
#si le mail vient de quelqu'un (de chez picasoft) qui s'est connecté avec un client SMTP (un humain ou mattermost) alors son mail n'aura pas de headers spf/dkim, ce qui fait qu'il est invalide au vu de notre propre politique dmarc. On trust donc tous les gens qui se sont connectés en sasl et on les force à pass le dmarc.
IgnoreAuthenticatedClients true
EOF

#configuration DKIM
#ecriture des tables indiquant quoi signer avec quelle clef
/opendkim-tables.sh
postconf -e "milter_default_action = accept"
postconf -e "milter_protocol = 6"
postconf -e "smtpd_milters = local:/opendkim/opendkim.sock, local:/opendmarc/opendmarc.sock"
postconf -e "non_smtpd_milters = local:/opendkim/opendkim.sock, local:/opendmarc/opendmarc.sock"

#création des répertoires où transiteront les sockets + gestion des permissions
mkdir /var/spool/postfix/opendkim
mkdir /var/spool/postfix/opendmarc
#mkdir /etc/opendmarc
chown -R opendmarc:opendmarc /etc/opendmarc
chown -R opendkim:opendkim /etc/opendkim.conf /etc/dkimkeys
chown opendmarc:opendmarc /etc/opendmarc.conf
chown opendkim:postfix /var/spool/postfix/opendkim
chown opendmarc:postfix /var/spool/postfix/opendmarc
adduser postfix opendkim
adduser postfix opendmarc


#SSL : récupération des certificats (qui seront utilisés pour la connexion smtp)
postconf -e "smtpd_tls_cert_file = ${SSL_CERT}"
postconf -e "smtpd_tls_key_file = ${SSL_KEY}"
postconf -e "smtpd_tls_security_level = may"

#autorise le SSL pour des mails sortant mais n'interdit pas le clair lorsque ce n'est pas disponible
postconf -e "smtp_tls_security_level=may"


#logs
#on redirige les logs des fichiers vers le stdout
rm /var/log/mail.log
rm /var/log/opendmarc.log
ln -s /dev/stdout /var/log/mail.log
ln -s /dev/stdout /var/log/opendmarc.log


#activation du service smtps (sur le port 465 par défaut)
cat <<EOF >> /etc/postfix/master.cf
smtps     inet  n       -       y       -       -       smtpd
submission inet n       -       y       -       -       smtpd
EOF

#workaround pour un problème de permission
touch /var/log/opendmarc.log
chown opendmarc:opendmarc /var/log/opendmarc.log

#configuration du socket TCP/IP, on est obligé d'utiliser ipv4 pour la local delivery car les docker network ne supportent pas iPv6 par défaut
postconf -e "inet_protocols = ipv4"
#enregistrement de cette instruction
postfix reload
#il est nécessaire de stopper le démon pour effectivement modifier les sockets TCP/IP (par défaut iPv6)
service postfix stop
#au cas où le démon postfix aurait été lancé indépendamment de systemd
postfix stop
